;
; Builds AutoIt3 help file
;

#Region Includes
#include "include\DocLib.au3"
#include "include\MiscLib.au3"
#include "include\SciTELib.au3"
#include "include\SettingsLib.au3"
#EndRegion Includes

#Region Global Variables
; The name of the project.
Global Const $g_sProjectLang = "english"
Global Const $g_sProject = "autoit3 help"
Global Const $g_sProjectDir = "docs\autoit"
#EndRegion Global Variables

#Region Constant Variables
Global Enum $HELPFILES_SOURCE, _
		$HELPFILES_DESTINATION, _
		$HELPFILE_SUCCESS, _
		$HELPFILES_MAX
#EndRegion Constant Variables

#Region Main body of code
Global $g_nExitCode = _Main()
Exit $g_nExitCode
#EndRegion Main body of code

#Region _Main()
; ===================================================================
; _Main()
;
; The main function responsible for generating the syntax files.
; Parameters:
;	None.
; Returns:
;	0 on success, non-zero on failure.
; ===================================================================
Func _Main()
	; Create the output window and initial message.
	_OutputWindowCreate()
	_OutputProgressWrite("==== Output for " & StringTrimRight(@ScriptName, StringLen(".exe")) & " (" & $g_sProject & ") ====" & @CRLF)
	_OutputProgressWrite("Generating AutoIt.chm..." & @CRLF)

	; Set the build directory based on the rules and the INI file value.
	Local $sBuildDir = _SettingBuildDir()

	; Get Setting to check if Rebuild all help files is required.
	Local $sRegenAll = ""
	If _SettingGet($SETTING_REBUILDHELPFILES, False, True, Default, True) Then $sRegenAll = "/RegenAll"

	; Delete files in the install dir that we are about to change.
	FileDelete("install\AutoIt.chm")

	; Update the helpfile.
	FileChangeDir($sBuildDir & "\" & $g_sProjectDir & "\" & $g_sProjectLang)
	RunWait('"' & @AutoItExe & '" All_Gen_AutoIt3.au3 ' & $sRegenAll)

	; Holds the return value.
	Local $iReturn = 0

	; Workaround for merging into one single chm file. By guinness.
	Local $hBackupTOC = _HelpFiles_Merge("AutoIt3 TOC.hhc", "UDFs3 TOC.hhc")
	Local $hBackupHHK = _HelpFiles_Merge("AutoIt3 Index.hhk", "UDFs3 Index.hhk")

	_OutputBuildWrite("Expand the Au3 code in the html files" & @CRLF)
	; Expand the Au3 code in the html files.
	Local $hHTMLFiles = _HTMLFiles_ExpandAu3Code()

	_OutputBuildWrite("Create helpfile" & @CRLF)
	; Create the helpfile.
	CompileDocumentation("AutoIt3.hhp")
	If @error Then
		_OutputProgressWrite("Error: Unable to compile documentation." & @CRLF)
		$iReturn = 1
	Else
		; Copy the files install
		FileChangeDir($sBuildDir)
		FileMove($g_sProjectDir & "\" & $g_sProjectLang & "\AutoIt3.chm", "install\AutoIt.chm", $FC_OVERWRITE) ; Move AutoIt3.chm to the install folder as AutoIt.chm.

		; Save Html file for upload to WEB.
		Local $sWEB_html_Dir = $g_sProjectDir & "\" & $g_sProjectLang & "\WEB_html"
		DirRemove($sWEB_html_Dir, 1)
		DirCopy($g_sProjectDir & "\" & $g_sProjectLang & "\html", $sWEB_html_Dir, $FC_OVERWRITE)

		; Delete all temp files ready for source code packaging
		FileDelete($g_sProjectDir & "\" & $g_sProjectLang & "\Debug.log")
		FileDelete($g_sProjectDir & "\" & $g_sProjectLang & "\_errorlog3.txt")
		FileDelete($g_sProjectDir & "\" & $g_sProjectLang & "\genindex.log")

		; Write closing message and wait for close (if applicable).
		_OutputProgressWrite("Finished." & @CRLF & @CRLF) ; Two CRLF"s in case of chained output.
	EndIf

	FileChangeDir($sBuildDir & "\" & $g_sProjectDir & "\" & $g_sProjectLang)

	_OutputBuildWrite("Revert the Au3 code in the html files")
	; Revert the changes.
	_HTMLFiles_UnExpandAu3Code($hHTMLFiles)

	; Move the backed up files back again, thus no files have to be applied to the ignore list.
	_HelpFiles_UnMerge($hBackupTOC)
	_HelpFiles_UnMerge($hBackupHHK)

	_OutputWaitClosed($iReturn)

	Return $iReturn
EndFunc   ;==>_Main
#EndRegion _Main()

; Merge the AutoIt3 and UDF related files into one single file.
Func _HelpFiles_Merge($sSource, $sDestination)
	; Create an array to store the source and destination file paths.
	Local $aReturn[$HELPFILES_MAX]
	$aReturn[$HELPFILES_SOURCE] = _GetFullPath($sSource)

	; Create a temporary filepath to backup the file to. This guarantees the filepath doesn't exist.
	$aReturn[$HELPFILES_DESTINATION] = _WinAPI_GetTempFileName(@TempDir)
	If FileCopy($aReturn[$HELPFILES_SOURCE], $aReturn[$HELPFILES_DESTINATION], $FC_OVERWRITE_CREATE) Then ; Backup the previous file.
		$sDestination = _GetFullPath($sDestination)
		Local $aArray = StringRegExp(FileRead($sDestination), _
				"(?s)(<UL>.+</UL>)", $STR_REGEXPARRAYGLOBALMATCH)
		If Not @error Then
			Local $sAutoIt3TOC = FileRead($aReturn[$HELPFILES_SOURCE])
			_StripEmptyLines($sAutoIt3TOC)
			_StripWhitespace($sAutoIt3TOC)

			; Add child file to the end of the parent.
			$sAutoIt3TOC = StringRegExpReplace($sAutoIt3TOC, _
					"(</UL>\R)(?=</BODY></HTML>)", "\1" & StringRegExpReplace($aArray[0], "([$\\])", "\\\1") & @CRLF) ; Escape \ and $ in the replacement data.
			_FileWrite($aReturn[$HELPFILES_SOURCE], $sAutoIt3TOC)
		EndIf
	Else ; An error occurred backing up.
		FileDelete($aReturn[$HELPFILES_DESTINATION]) ; Delete the file as _WinAPI_GetTempFileName() creates the file automatically.
		$aReturn[$HELPFILES_DESTINATION] = Null ; Set to Null to check with _HelpFiles_UnMerge().
	EndIf
	Return $aReturn
EndFunc   ;==>_HelpFiles_Merge

; Revert the previous backup file.
Func _HelpFiles_UnMerge(ByRef $aHelpFiles)
	Local $bReturn = False
	If UBound($aHelpFiles) = $HELPFILES_MAX And $aHelpFiles[$HELPFILES_DESTINATION] <> Null Then ; If the array is the correct size and the destination element isn't null.
		$bReturn = FileMove($aHelpFiles[$HELPFILES_DESTINATION], $aHelpFiles[$HELPFILES_SOURCE], $FC_OVERWRITE_CREATE) > 0
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpFiles_UnMerge

; Expand the Au3 syntax in the html files. The code is between @@SyntaxHighlighting@@ @CRLF CODE @@End@@ @CRLF
Func _HTMLFiles_ExpandAu3Code()
	; Create a list of all files in the udf_devs\docs\autoit\english\html folder and exclude the auto generate folders.
	Local $aFileList = _FileListToArrayRec(_GetFullPath("html"), "*.htm||functions;keywords;libfunctions", $FLTAR_FILES, $FLTAR_RECUR, Default, $FLTAR_FULLPATH)
	If @error Then Return Null

	; Set the SciTE directory.
	_SciTE_SetSciTEDir(@ScriptDir & "\..\..\install\SciTe")

	; Ensure SciTE is not running and then load with the default properties.
	Local Const $bIsSciTERunningBefore = _SciTE_Run()

	Local Enum $eCodeSnippet_Full, $eCodeSnippet_Code, $eCodeSnippet_Max
	Local $aCodeSnippets = 0, $aHTMLFiles[$aFileList[0]][$HELPFILES_MAX], _
			$iIndex = 0, _
			$sAu3Script = "", $sCodeSnippet = "", $sData = "", $sTempFilePath = ""

	; Create a temporary Au3 script.
	Do
		$sTempFilePath = _WinAPI_GetTempFileName(@TempDir)
		$sAu3Script = StringRegExpReplace($sTempFilePath, "(?:\.\w+$)", ".au3") ; Replace the extension to .au3 for SciTE to highlight the code correctly.
		FileDelete($sTempFilePath) ; Delete the temporary file.
	Until Not FileExists($sAu3Script)
	_FileWrite($sAu3Script, "", $FO_UTF8) ; Create the temporary file for storing the Au3 script.

	$sTempFilePath = _WinAPI_GetTempFileName(@TempDir) ; Create a new temporary file.

	For $i = 1 To UBound($aFileList) - 1
		$aHTMLFiles[$iIndex][$HELPFILES_SOURCE] = $aFileList[$i]
		$sData = FileRead($aHTMLFiles[$iIndex][$HELPFILES_SOURCE])
		If Not @error And StringInStr($sData, "@@SyntaxHighlighting@@") Then ; If FileRead() didn't return an error and @@SyntaxHighlighting@@ exists in the file then parse the snippets.
			; Create a temporary filepath to backup the file to. This guarantees the filepath doesn"t exist.
			$aHTMLFiles[$iIndex][$HELPFILES_DESTINATION] = _WinAPI_GetTempFileName(@TempDir)
			If FileCopy($aHTMLFiles[$iIndex][$HELPFILES_SOURCE], $aHTMLFiles[$iIndex][$HELPFILES_DESTINATION], $FC_OVERWRITE_CREATE) Then ; Backup the previous file.

				$aCodeSnippets = StringRegExp($sData, "(?is)(@@SyntaxHighlighting@@\h*\R(.+?)\R@@End@@(?:\h*\R)*)", $STR_REGEXPARRAYGLOBALMATCH) ; Parse the code looking for the syntax highlighting tags.
				If Not @error Then
					For $j = 0 To UBound($aCodeSnippets) - 1 Step $eCodeSnippet_Max
						$sCodeSnippet = $aCodeSnippets[$j + $eCodeSnippet_Code]
						If _FileWrite($sAu3Script, $sCodeSnippet, $FO_UTF8) Then ; Write the code snippet to the temporary file.
							$sCodeSnippet = _SciTE_Au3ToHTML($sAu3Script, $sTempFilePath, True) ; Don't expand the links.
							$sCodeSnippet = StringRegExpReplace($sCodeSnippet, "\R", "<br />") ; This is a workaround for EOL characters not being converted to <br />. Generally speaking this works.
							$sData = StringReplace($sData, $aCodeSnippets[$j + $eCodeSnippet_Full], _ ; Replace the full string with the HTML Au3 code.
									'<p class="codebox">' & @CRLF & $sCodeSnippet & @CRLF & "</p>" & @CRLF, $STR_REPLACE_ONCE, $STR_CASESENSE)
						EndIf
					Next
					_FileWrite($aHTMLFiles[$iIndex][$HELPFILES_SOURCE], $sData, $FO_UTF8) ; Write the expanded Au3 code to the html file.
					$iIndex += 1 ; Increase the index.
				EndIf
			Else ; An error occurred backing up.
				FileDelete($aHTMLFiles[$iIndex][$HELPFILES_DESTINATION]) ; Delete the file as _WinAPI_GetTempFileName() creates the file automatically.
			EndIf
		EndIf
	Next
	ReDim $aHTMLFiles[$iIndex][$HELPFILES_MAX] ; Re-size the array to the correct size.

	If $bIsSciTERunningBefore Then
		; Close the open files.
		_SciTE_CloseAll()
		; Reload the saved session.
		_SciTE_LoadSession(_SciTE_GetSessionPath())
		; Reload the SciTE properties.
		_SciTE_ReloadProperties()
	Else
		_SciTE_Quit()
	EndIf

	; Close the SciTE processing functions.
	_SciTE_Shutdown()

	; Delete the temporary files.
	FileDelete($sAu3Script)
	FileDelete($sTempFilePath)

	Return $aHTMLFiles
EndFunc   ;==>_HTMLFiles_ExpandAu3Code

; Revert the previous backup files.
Func _HTMLFiles_UnExpandAu3Code(ByRef $aHTMLFiles)
	Local $bReturn = False
	If UBound($aHTMLFiles, $UBOUND_COLUMNS) = $HELPFILES_MAX Then ; If the array is the correct size.
		$bReturn = True
		For $i = 0 To UBound($aHTMLFiles) - 1
			$aHTMLFiles[$i][$HELPFILE_SUCCESS] = FileMove($aHTMLFiles[$i][$HELPFILES_DESTINATION], $aHTMLFiles[$i][$HELPFILES_SOURCE], $FC_OVERWRITE_CREATE)
			If Not $aHTMLFiles[$i][$HELPFILE_SUCCESS] Then
				$bReturn = False
				FileDelete($aHTMLFiles[$i][$HELPFILES_DESTINATION]) ; Delete the file as we don"t want temporary files being left.
			EndIf
		Next
	EndIf
	Return $bReturn
EndFunc   ;==>_HTMLFiles_UnExpandAu3Code
